@@ -34,11 +34,9 @@
 #include <sys/socket.h>    /* socket() */
 #include <sys/un.h>        /* struct sockaddr_un */
 
-#ifdef linux
 #include <linux/vt.h>      /* VT_GETSTATE */
 #include <sys/kd.h>        /* KDGETMODE */
 #include <termios.h>       /* winsize */
-#endif
 
 #include "gpmInt.h"
 
@@ -48,7 +46,7 @@
 
 extern int	errno;
 
-static void *gpm_killed(int);
+static void gpm_killed(int);
 
 /*-------------------------------------------------------------------*/
 char *opt_type=DEF_TYPE;
@@ -65,6 +63,8 @@ int opt_cluster=DEF_CLUSTER;
 int opt_three=DEF_THREE;
 int opt_dirty=DEF_DIRTY;
 int opt_test=DEF_TEST;
+int opt_ptrdrag=DEF_PTRDRAG;
+int opt_kill=0;
 
 char *prgname;
 Gpm_Type *m_type;
@@ -93,7 +93,6 @@ int fd;
 }
 
 /*-------------------------------------------------------------------*/
-#ifdef linux
 static inline int check_mode(int *fdptr)
 {
 int fd, ch = 0;
@@ -111,14 +110,15 @@ long kd_mode;
     sleep(2); ch++;
     }
 
-  if (ch && (*fdptr=open(opt_dev,O_RDWR))<0)
-    oops(opt_dev);
+  if (ch) {
+    if ((*fdptr=open(opt_dev,O_RDWR))<0)
+      oops(opt_dev);
+    if (m_type->init)
+      (m_type->init)(*fdptr, m_type->flags, m_type);
+  }
 
   return (ch>0);
 }
-#else
-static inline int check_mode(int *fdptr) {return 0;}
-#endif
 
 /*-------------------------------------------------------------------*/
 static inline void selection_copy(int x1, int y1, int x2, int y2, int mode)
@@ -139,16 +139,12 @@ int fd;
   arg[3]=(unsigned short)y2;
   arg[4]=(unsigned short)mode;
 
-#ifdef linux
   if ((fd=open_console(O_WRONLY))<0)
     oops("open_console");
   DEBUGG((stderr,"ctl %i, mode %i\n",(int)*buf,arg[4]));
   if (ioctl(fd, TIOCLINUX, buf+sizeof(short)-1) < 0)
     oops("ioctl(TIOCLINUX)");
   close(fd);
-#else
-  if (mode != 3) fprintf(stderr,"copy ");
-#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -157,21 +153,19 @@ static inline void selection_paste(void)
 char c=3;
 int fd;
 
-#ifdef linux
   fd=open_console(O_WRONLY);
   if (ioctl(fd, TIOCLINUX, &c) < 0)
      oops("ioctl(TIOCLINUX)");
   close(fd);
-#else
-  fprintf(stderr,"paste ");
-#endif
 }
 
 /*-------------------------------------------------------------------*/
 static  inline int do_selection(Gpm_Event *event)  /* returns 0, always */
 {
 static int vc=0;
-static int x1, y1, x2, y2;
+static int x1=1, y1=1, x2, y2;
+static int ptr=0;
+#define UNPOINTER() 0
 
   x2=event->x; y2=event->y;
   switch(GPM_BARE_EVENTS(event->type))
@@ -194,6 +188,8 @@ static int x1, y1, x2, y2;
 	    case GPM_LFT: y2<=y1 ? x2++ : (x2=maxx, y2--); break;
 	    }
         selection_copy(x1,y1,x2,y2,event->clicks);
+	if (event->clicks>=opt_ptrdrag && !event->margin) /* pointer */
+	  selection_copy(x2,y2,x2,y2,3);
 	}
       return 0;
 
@@ -223,17 +219,15 @@ static int x1, y1, x2, y2;
 
 /*-------------------------------------------------------------------*/
 /* returns 0 if the event has not been processed, and 1 if it has */
-static inline int do_client(int fd, Gpm_Event *event)
+static inline int do_client(Gpm_Cinfo *cinfo, Gpm_Event *event)
 {
 int vc=event->vc;
-Gpm_Connect info=cinfo[vc]->data;
+Gpm_Connect info=cinfo->data;
+int fd=cinfo->fd;
 /* value to return if event is not used */
 int res = !(info.defaultMask & event->type); 
 MAGIC_P((static int magic=GPM_MAGIC));
 
-  /* first, check if the event is used by the client */
-  info=cinfo[vc]->data;
-
   /* control keys: pass it over */
   if ((info.minMod & event->modifiers) < info.minMod) return 0;
   if ((info.maxMod & event->modifiers) < event->modifiers) return 0;
@@ -257,10 +251,11 @@ static inline char *getMouseData(int fd, Gpm_Type *m_type)
 {
 static unsigned char data[32]; /* quite a big margin :) */
 char *edata=data+(m_type->proto)[4];
+int howmany=m_type->howmany;
 int i,j;
 /*....................................... read and identify one byte */
 
-    if (read(fd, data, 1)!=1)
+    if (read(fd, data, howmany)!=howmany)
       {
       if (opt_test) exit(0);
       LOG(("Error in read()ing first"));
@@ -278,13 +273,14 @@ int i,j;
      * well, this seems to work almost right with ps2 mice. However, I've never
      * tried ps2 with the original selection package, which called usleep()
      */
-    i=(m_type->proto)[4]-1; /* still to get */
-    do
-      {
-      j=read(fd,edata-i,i); /* edata is pointer just after data */
-      i-=j;
-      }
-    while (i&&j);
+    
+    if (i=(m_type->proto)[4]-howmany) /* still to get */
+      do
+	{
+	j=read(fd,edata-i,i); /* edata is pointer just after data */
+	i-=j;
+	}
+      while (i&&j);
 
     if (i)
       {
@@ -307,9 +303,7 @@ char *data;
 static int fine_dx, fine_dy;
 static int i, margin, j;
 static Gpm_Event nEvent;
-#ifdef linux
 static struct vt_stat stat;
-#endif
 static int clicks=0;    /* 0,1 or 2 */
 static struct timeval tv1={0,0}, tv2; /* tv1=0: force first click as single */
 static struct timeval timeout={0,0};
@@ -336,8 +330,6 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
 
     FD_ZERO(&fdSet); FD_SET(fd,&fdSet); i=0; do /* cluster loop */
       {
-/*      if (i) DEBUG((stderr,"C"));*/
-
       if (   ((data=getMouseData(fd,m_type))==NULL)
 	  || ((*(m_type->fun))(&nEvent,data)==-1) )
 	{
@@ -345,7 +337,6 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
 	else break;
 	}
 
-
       nEvent.buttons = opt_sequence[nEvent.buttons]&7; /* change the order */
       oldB=newB; newB=nEvent.buttons;
       if (!i) event->buttons=nEvent.buttons;
@@ -381,7 +372,6 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
 
   event->x+=event->dx, event->y+=event->dy;
 
-#ifdef linux
   i=open_console(O_RDONLY);
   ioctl(i,VT_GETSTATE,&stat);
   event->modifiers=6; /* code for the ioctl */
@@ -389,10 +379,6 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
     oops("get_shift_state");
   close(i);
   event->vc = stat.v_active;
-#else
-  event->vc=0;
-  event->modifiers=0;
-#endif
 
   if (oldB==event->buttons)
     event->type = (event->buttons ? GPM_DRAG : GPM_MOVE);
@@ -410,7 +396,7 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
 
     case GPM_UP:
       GET_TIME(tv1);
-      event->buttons=oldB; /* for button-up, tell which one */
+      event->buttons^=oldB; /* for button-up, tell which one */
       event->type|= (GPM_SINGLE<<clicks);
       event->type|= (oldT&GPM_MFLAG);
       break;
@@ -576,14 +562,11 @@ struct sockaddr_un addr; /* reuse this each time */
        request->minMod, request->maxMod));
 
 /* if the client gets motions, give it the current position */
-  WD;
   if(request->eventMask & GPM_MOVE)
     {
     Gpm_Event event={0,0,vc,0,0,mouseX,mouseY,GPM_MOVE,0,0};
-    do_client(newfd, &event);
+    do_client(info, &event);
     }
-  WD;
-
 
   return newfd;
 }
@@ -606,40 +589,13 @@ Gpm_Event event;
   mousefd=cmdline(argc, argv);
   maxfd=max(mousefd,maxfd);
 
-/*....................................... check for uniqueness and sign */
-
-  if (chdir(GPM_NODE_DIR) && mkdir(GPM_NODE_DIR,GPM_NODE_DIR_MODE))
-    oops(GPM_NODE_DIR);
-  if (chdir(GPM_NODE_DIR))
-    oops(GPM_NODE_DIR);      /* well, I must create my directory first */
-
-  f=fopen(GPM_NODE_PID,"r");
-  if (f && fscanf(f,"%i",&pid)==1 && kill(pid,0)==0 && pid!=getpid())
-    {
-    if (opt_dirty) kill(pid,9);
-    else
-      {
-      fprintf(stderr,"gpm-Linux is already running as pid %i\n",pid);
-      exit(1);
-    } }
-  if (f) fclose(f);
-  LOG(("Alive and well"));
-
-  f=fopen(GPM_NODE_PID,"w");
-  if (!f) oops(GPM_NODE_PID);
-  fprintf(f,"%i\n",getpid());
-  fclose(f);
-  LOG(("Signed"));
-
   signal(SIGTERM,gpm_killed);
+  signal(SIGUSR1,gpm_killed); /* usr1 is used by a new gpm killing the old */
   signal(SIGINT, gpm_killed);
 
 /*....................................... create your nodes */
 
   /* control node */
-  ctladdr.sun_family=AF_UNIX;
-  unlink (GPM_NODE_CTL);
-  strcpy(ctladdr.sun_path,GPM_NODE_CTL);
 
   if ((ctlfd=socket(AF_UNIX,SOCK_STREAM,0))==-1)
     oops("socket()");
@@ -728,8 +684,8 @@ Gpm_Event event;
 	 * or to the default handler, if any
 	 * or to the selection handler
 	 */
-        (cinfo[event.vc] && do_client(cinfo[event.vc]->fd, &event))
-        || (cinfo[0]     && do_client(cinfo[0]->fd, &event))
+        (cinfo[event.vc] && do_client(cinfo[event.vc], &event))
+        || (cinfo[0]     && do_client(cinfo[0], &event))
         || do_selection(&event);
       }
 
@@ -775,10 +731,26 @@ Gpm_Event event;
 }
 
 /*-------------------------------------------------------------------*/
-static void *gpm_killed(int signo)
+static void gpm_killed(int signo)
 {
-  DEBUG((stderr,"Removing files...\n"));
-  unlink(GPM_NODE_PID);
-  unlink(GPM_NODE_CTL);
+  if (signo==SIGUSR1)
+    fprintf(stderr,"%s pid %i killed by a new %s\n",prgname,getpid(),prgname);
+  else
+    {
+    DEBUG((stderr,"Removing files...\n"));
+    unlink(GPM_NODE_PID);
+    unlink(GPM_NODE_CTL);
+    }
   exit(0);
 }
+
+
+/*
+ * This is because Linus uses 4-wide tabstops,
+ * forcing me to use the same default to manage kernel sources
+ */
+
+/* Local Variables: */
+/* tab-width:8      */
+/* End:             */
+
