@@ -19,8 +19,8 @@
  ********/
 
 #define GPM_NAME    "gpm-Linux"
-#define GPM_RELEASE "0.9"
-#define GPM_DATE    "October 1994"
+#define GPM_RELEASE "0.97"
+#define GPM_DATE    "January 1995"
 
 /* You can define DEBUG, DEBUGG, or neither. */
 /* Well, better putting them in the Makefile... */
@@ -56,4 +56,5 @@
 
 #define DEF_DIRTY            0
 #define DEF_TEST             0
+#define DEF_PTRDRAG         10    /* never */
 
