@@ -45,11 +45,11 @@ extern int Log(char *fmt, ...);
  */
 typedef struct Gpm_Type {
   char *name;
-  char *device;
   int (*fun)(Gpm_Event *state, unsigned char *data);
-  int (*init)(int fd, unsigned short flags);
+  int (*init)(int fd, unsigned short flags, struct Gpm_Type *type);
   unsigned short flags;
   unsigned char proto[5];
+  int howmany; /* howmany bytes to read at a time */
 }                   Gpm_Type;
 
 
@@ -74,7 +74,8 @@ extern char *prgname;
 extern char *opt_type, *opt_lut, *opt_dev;
 extern char *opt_sequence;
 extern int opt_baud, opt_sample, opt_delta, opt_accel, opt_scale;
-extern int opt_dirty, opt_test, opt_time, opt_cluster, opt_three;
+extern int opt_dirty, opt_test, opt_time, opt_cluster, opt_three, opt_ptrdrag;
+extern int opt_kill;
 
 extern Gpm_Type mice[];
 extern Gpm_Type *m_type;
