@@ -26,9 +26,13 @@
 #include <stdarg.h>        /* Log uses it */
 #include <errno.h>
 #include <unistd.h>        /* getopt() */
+#include <sys/stat.h>      /* mkdir()  */
 #include <sys/param.h>
 #include <sys/time.h>      /* timeval */
 #include <sys/wait.h>      /* wait() */
+#include <sys/types.h>     /* socket() */
+#include <sys/socket.h>    /* socket() */
+#include <sys/un.h>        /* struct sockaddr_un */
 
 #ifdef	SIGTSTP		/* true if BSD system */
 #include <sys/file.h>
@@ -37,7 +41,6 @@
 
 #include "gpmInt.h"
 #include "gpm.h"        /* this includes socket and internet headers */
-#include "getopt.h"     /* uninfluent for linux, needed in sunos */
 
 extern int	errno;
 
@@ -111,16 +114,19 @@ static unsigned long long_array[5]={
     else SET(c);
     }
 
-#ifdef linux
   if ((fd=open("/dev/console", O_WRONLY)) < 0)
     oops("/dev/console");
   if ((fd<0) || (ioctl(fd, TIOCLINUX, &long_array)<0))
     {
-    fprintf(stderr,"Is your kernel compiled with CONFIG_SELECTION on?\n");
+    if (getuid())
+      fprintf(stderr,"%s: you're not root, can you ioctl(TIOCLINUX)?\n",
+	      prgname);
+    else
+      fprintf(stderr,"%s: is your kernel compiled with CONFIG_SELECTION on?\n",
+	      prgname);
     oops("loadlut");
     }
   close(fd);
-#endif
 
   return 0;
 }
@@ -137,9 +143,12 @@ static int usage(void)
  "    -d delta         sets the delta value (default %d)\n"
  "    -D               dirty operation (debug only)\n"
  "    -i interval      max time interval for multiple clicks (default %i)\n"
+ "    -k               kill a running gpm, to start X with a busmouse\n"
  "    -l charset       loads the inword() LUT (default '%s')\n"
  "    -L charset       load LUT and exit\n"
  "    -m mouse-device  sets mouse device\n"
+ "    -p               draw the pointer while striking a selection\n"
+ "    -P               do it only with word and line drags\n"
  "    -r number        sets the responsiveness (default %i)\n"
  "    -s sample-rate   sets the sample rate (default %d)\n"
  "    -t mouse-type    sets mouse type (default '%s')\n"
@@ -156,7 +165,7 @@ static int usage(void)
 /*===================================================================*/
 int cmdline(int argc, char **argv)
 {
-char options[]="a:b:B:d:Dhi:l:L:m:r:s:t:Tv23";
+char options[]="a:b:B:d:Dhi:kl:L:m:pPr:s:t:Tv23";
 int opt, fd;
 static struct {char *in; char *out;} seq[] = {
   {"123","01234567"},
@@ -172,58 +181,31 @@ static struct {char *in; char *out;} seq[] = {
     {
     switch (opt)
       {
-      case 'a':
-          opt_accel = atoi(optarg);
-          break;
-      case 'b':
-          opt_baud = atoi(optarg);
-          break;
-      case 'B':
-	  opt_sequence = optarg;
-	  break;
-      case 'd':
-          opt_delta = atoi(optarg);
-          break;
-      case 'D':
-          opt_dirty++;
-	  break;
-      case 'h':
-	  exit(usage());
-      case 'i':
-	  opt_time=atoi(optarg);
-	  break;
-      case 'l': /* load look-up-table */
-          opt_lut=optarg;
-          break;
-      case 'L':
-	  loadlut(optarg);
-	  exit(0);
-      case 'm':
-          opt_dev = optarg;
-          break;
+      case 'a': opt_accel = atoi(optarg); break;
+      case 'b': opt_baud = atoi(optarg); break;
+      case 'B': opt_sequence = optarg; break;
+      case 'd': opt_delta = atoi(optarg); break;
+      case 'D': opt_dirty++; break;
+      case 'h': exit(usage());
+      case 'i': opt_time=atoi(optarg); break;
+      case 'k': opt_kill++; break;
+      case 'l': opt_lut=optarg; break;
+      case 'L': loadlut(optarg); exit(0);
+      case 'm': opt_dev = optarg; break;
+      case 'p': opt_ptrdrag=0; break;
+      case 'P': opt_ptrdrag=1; break;
       case 'r':
 	  /* being called responsiveness, I must take the inverse */
 	  opt_scale=atoi(optarg);
           if (!opt_scale) opt_scale=100; /* a maximum */
 	  else opt_scale=100/opt_scale;
 	  break;
-      case 's':
-          opt_sample = atoi(optarg);
-          break;
-      case 't':
-          opt_type=optarg;
-	  break;
-      case 'T':
-	  opt_test++;
-	  break;
-      case 'v':
-	  printf(GPM_NAME " " GPM_RELEASE ", " GPM_DATE "\n"); exit(0);
-      case '2':
-          opt_three=-1;
-          break;
-      case '3':
-	  opt_three=1;
-	  break;
+      case 's': opt_sample = atoi(optarg); break;
+      case 't': opt_type=optarg; break;
+      case 'T': opt_test++; break;
+      case 'v': printf(GPM_NAME " " GPM_RELEASE ", " GPM_DATE "\n"); exit(0);
+      case '2': opt_three=-1; break;
+      case '3': opt_three=1; break;
       default:
           fprintf(stderr,"%s: try \"%s -h\" to get help\n",
 		  prgname, prgname);
@@ -242,27 +224,98 @@ static struct {char *in; char *out;} seq[] = {
   if (!(m_type->fun)) /* not found */
     exit(usage());
 
+
+
+  /* check for uniqueness and sign */
+
+  if (chdir(GPM_NODE_DIR) && mkdir(GPM_NODE_DIR,GPM_NODE_DIR_MODE))
+    oops(GPM_NODE_DIR);
+  if (chdir(GPM_NODE_DIR))
+    oops(GPM_NODE_DIR);      /* well, I must create my directory first */
+
+  /* this is the new authentication */
+  {
+  struct sockaddr_un ctladdr;
+  int ctlfd, pid, len; FILE *f;
+  
+  bzero((char *)&ctladdr,sizeof(ctladdr));
+  ctladdr.sun_family=AF_UNIX;
+  strcpy(ctladdr.sun_path, GPM_NODE_CTL);
+  len=sizeof(ctladdr.sun_family)+strlen(GPM_NODE_CTL);
+
+  if ( (ctlfd=socket(AF_UNIX,SOCK_STREAM,0))<0 )
+	oops("socket()");
+
+  if ( connect(ctlfd,(struct sockaddr *)(&ctladdr),len)>=0 )
+	{
+	/* another gpm is runnin, get its pid */
+	f=fopen(GPM_NODE_PID,"r");
+	if (f && fscanf(f,"%i",&pid)==1)
+	  {
+	  if (opt_dirty || opt_kill) 
+	    {
+	    if (kill(pid,SIGUSR1)==-1) oops("kill()");
+	    if (opt_kill) exit(0);
+	    }
+	  else
+	    {
+	    fprintf(stderr,"gpm-Linux is already running as pid %i\n",pid);
+	    exit(1);
+	    }
+	  }
+	else
+	  {
+	  fprintf(stderr,"%s: fatal error (should't happen)\n",prgname);
+	  exit(1);
+	  }
+	}
+
+  close(ctlfd);
+
+  /* go to background and become a session leader (Stefan Giessler) */
+
+  switch(fork())
+    {
+    case -1: oops("fork()"); /* error */
+    case  0: break;          /* child */
+    default: exit(0);        /* parent */
+    }
+
+  if (!freopen("/dev/console","w",stderr))
+    {
+    printf("gpm: freopen(stderr) failed\n"); exit(1);
+    }
+  if (setsid()<0) oops("setsid()");
+
+  /* now sign */
+  f=fopen(GPM_NODE_PID,"w");
+  if (!f)
+    {
+    if (getuid()) fprintf(stderr,"%s: you're not root, can you write to %s?\n",
+			  prgname,GPM_NODE_DIR);
+    oops(GPM_NODE_PID);
+    }
+  fprintf(f,"%i\n",getpid());
+  fclose(f);
+  LOG(("Signed"));
+  }
+
   /* open the device */
   if (opt_dev)
     {
     if ((fd=open(opt_dev,O_RDWR))<0)
       oops(opt_dev); /* user failed */
     }
-  else /* use "/dev/mouse", and the the type-specific name */
+  else /* use "/dev/mouse" */
     {
     opt_dev = "/dev/mouse";
     if ((fd=open(opt_dev,O_RDWR))<0)
-      {
-      DEBUG((stderr,"%s: %s: %s\n",prgname,opt_dev,strerror(errno)));
-      opt_dev = m_type->device;
-      if ((fd=open(opt_dev,O_RDWR))<0)
-        oops(m_type->device); /* default failed */
-      }
+      oops("/dev/mouse");
     }
 
   /* init the device */
   if (m_type->init)
-    (m_type->init)(fd, m_type->flags);
+    (m_type->init)(fd, m_type->flags, m_type);
 
   loadlut(opt_lut);
 
@@ -272,13 +325,17 @@ static struct {char *in; char *out;} seq[] = {
   if (!seq[opt].in) exit(usage());
   opt_sequence=seq[opt].out;
 
-#ifndef linux
-  /* make the mouse non blocking... */
-  if ((opt=fcntl(fd,F_GETFL,0))<0)  oops("fcntl()");
-  if (fcntl(fd,F_SETFL,opt|O_NONBLOCK)<0) oops("fcntl()");
-#endif
-
-
   return fd;
 }
   
+
+/*
+ * This is because Linus uses 4-wide tabstops,
+ * forcing me to use the same default to manage kernel sources
+ */
+
+/* Local Variables: */
+/* tab-width:8      */
+/* End:             */
+
+
