@@ -35,11 +35,9 @@
 #include <sys/ioctl.h>
 #endif
 
-#ifdef linux
 #include <linux/vt.h>      /* VT_GETSTATE */
 #include <sys/kd.h>        /* KDGETMODE */
 #include <termios.h>       /* winsize */
-#endif
 
 
 #include "gpmInt.h"
@@ -135,22 +133,13 @@ Gpm_Stst *new;
     {
     conn->vc=0;                 /* default handler */
     if (flag>0) conn->vc=flag;  /* forced vc number */
-    #ifdef linux
     if (flag==0)  /* use your current vc */
       {
       tty=ttyname(0); /* stdin */
-      if (!tty || strncmp(tty,"/dev/tty",8))
+      if (!tty || strncmp(tty,"/dev/tty",8) || !isdigit(tty[8]))
 	goto err;
-      if (isdigit(tty[8]))
-        conn->vc=atoi(tty+8);
-      else
-	{
-	sscanf(tty+9,"%x",&(conn->vc));
-	conn->vc+=(tty[8]-'o')<<8;
-	DEBUGG((stderr,"%s is vc %i\n",tty,conn->vc));
-	}
+      conn->vc=atoi(tty+8);
       }
-    #endif
     }
 
   new->info=*conn;
@@ -305,7 +294,6 @@ static int count;
   max = (gpm_fd>fd) ? gpm_fd : fd;
 
 /*...................................................................*/
-#ifdef linux
   if (gpm_fd>=0)                                            /* linux */
     while(1)
       {
@@ -335,7 +323,6 @@ static int count;
 	}
       }
   else
-#endif
 /*...................................................................*/
   if (gpm_fd==-2)                                           /* xterm */
     {
@@ -446,7 +433,7 @@ int c;
       if (c==3)
 	{
 	ePtr->type = GPM_UP | (GPM_SINGLE<<clicks);
-	ePtr->buttons = 0;
+	/* ePtr->buttons = 0; */ /* no, keep info from press event */
 	GET_TIME (tv1);
 	clicks = 0;
 	}
@@ -460,15 +447,16 @@ int c;
 	
 	switch (c)
 	  {
-	  case 0: ePtr->buttons|=GPM_B_LEFT;   break;
-	  case 1: ePtr->buttons|=GPM_B_MIDDLE; break;
-	  case 2: ePtr->buttons|=GPM_B_RIGHT;  break;
+	  case 0: ePtr->buttons=GPM_B_LEFT;   break;
+	  case 1: ePtr->buttons=GPM_B_MIDDLE; break;
+	  case 2: ePtr->buttons=GPM_B_RIGHT;  break;
 	  default:    /* Nothing */          break;
 	  }
 	}
       /* Coordinates are 33-based */
       /* Transform them to 1-based */
-      ePtr->x = mdata[1]-32;
-      ePtr->y = mdata[2]-32;
+      ePtr->x = mdata[1]-32-gpm_zerobased;
+      ePtr->y = mdata[2]-32-gpm_zerobased;
       return 0;
 }
+
