@@ -48,8 +48,8 @@
 #endif
 
 #define RMEV_NAME "Mev"
-#define RMEV_RELEASE "0.9"
-#define RMEV_DATE "October 1994"
+#define RMEV_RELEASE "0.97"
+#define RMEV_DATE "December 1994"
 
 
 /*===================================================================*/
@@ -107,7 +107,7 @@ char cmd[128];
 
   gpm_handler = user_handler;
 
-  fprintf(stderr,"This program prints on stdout any event it"
+  fprintf(stderr,"This program prints on stdout any event it "
 	  "gets from stdin and from the mouse\r\n");
 
   while ((c=Gpm_Getc(stdin))!=EOF)
