@@ -123,22 +123,20 @@ by 't-mouse.el")
 
 
 ;; Restore normal mouse behaviour outside Emacs.
-(defun t-mouse-suspend () (process-send-string t-mouse-process "push -e0\n"))
-(defun t-mouse-resume () (process-send-string t-mouse-process "pop\n"))
+(defun t-mouse-suspend ()
+  (process-send-string t-mouse-process "push -e 0 -d any\n"))
+(defun t-mouse-resume ()
+  (process-send-string t-mouse-process "pop\n"))
 
 (add-hook 'suspend-hook 't-mouse-suspend)
 (add-hook 'suspend-resume-hook 't-mouse-resume)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; and the the processing commands
-(defun t-mouse-set-point (click)
-  (interactive "e")
-  (goto-char (nth 1 click)))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; make your keymap
 
 
 (defvar t-mouse-keymap nil "Keymap used by t-mouse")
 (defvar t-mouse-scroll nil "The last position when scrolling")
+(defvar t-mouse-word-syntax "w_" "The syntax set to select with double-click")
 
 (setq t-mouse-keymap
       '(keymap
@@ -188,8 +186,8 @@ by 't-mouse.el")
   (let* ((lines (+ (count-lines (point-min) (point-max)) (window-height)))
 	 (target (/ (* lines (cdr (nth 2 event))) (window-height)))
 	 (step (/ lines (window-height))))
-    (goto-line target)
-    (recenter)
+;;    (goto-line target)
+;;    (recenter)
     (setq t-mouse-scroll (cons step  (cdr (nth 2 event))))
 ))
 
@@ -226,6 +224,44 @@ by 't-mouse.el")
        (goto-char current) 
        (tm-copy event)))))
 
+(defun tm-word (event)
+  (let* ((current (nth 1 event))
+	(previous (mark))
+	(backward (> previous current)))
+    (pop-mark)
+    ;; update previous and set mark
+    (goto-char previous)
+    (if backward (skip-syntax-forward  t-mouse-word-syntax)
+      (skip-syntax-backward t-mouse-word-syntax))
+    (setq previous (point))
+    (push-mark (point) 'nomsg 'activate)
+    (sit-for 0.5)
+    ;; update current
+    (goto-char current)
+    (if backward (skip-syntax-backward t-mouse-word-syntax) 
+      (skip-syntax-forward  t-mouse-word-syntax))
+    (setq current (point))
+    (kill-ring-save (mark) current)))
+  
+(defun tm-line (event)
+  (let* ((current (nth 1 event))
+	(previous (mark))
+	(backward (> previous current)))
+    (tm-drag event)
+    (pop-mark)
+    ;; update previous and set mark
+    (goto-char previous)
+    (if backward (end-of-line) (beginning-of-line))
+    (setq previous (point))
+    (push-mark (point) 'nomsg 'activate)
+    (sit-for 0.5)
+    ;; update current
+    (goto-char current)
+    (if backward (beginning-of-line) (end-of-line))
+    (setq current (point))
+    (kill-ring-save (mark) current)))
+
+
 (defun tm-yank (event)
   (let ((current (nth 1 event)))
     (or
